#!/usr/bin/env python3

km = 1.e3
cm = 1.e-2
erg = 1.e-7
gram = 1.e-3
day = 86400.
yr = 365.25*day
EPS = 1.0e-38

from math import log10

def read_sfd(filename):

  f = open(filename, "r")

  D = []
  N = []
  for line in f.readlines():
    l = line.split()
    pop = int(l[1])
    if pop != 1:
      continue
    D.append(float(l[3])*cm)
    N.append(float(l[4])/float(l[2]))  # mtot/mbin

  f.close()

  return D, N

def write_sfd(D, N, filename):

  f = open(filename, "w")
  f.write("# D N\n")
  f.write("# km 1\n")
  for i in range(0,len(D)):
    f.write("%e %e\n" % (D[i]/km, N[i]))
  f.close()

def Qstar(D):
  """Scaling law, in cgs."""
  rho = 3.0e0
  Q0  = 9.0e7
  a   = -0.53e0
  B   = 0.5e0
  b   = 1.36e0

  return (Q0*(D/cm/2.)**a + B*rho*(D/cm/2.)**b) * (erg/gram)

def loginterp(x, y, x0):
  """Logarithmic interpolation"""

  if x0 < x[0]:
    extra = 1
    i = 1
  elif x0 > x[-1]:
    extra = 1
    i = len(x)-1
  else:
    extra = 0
    i = 1
    while x0 > x[i]:
      i += 1

  print("i = ", i)
  print("x0 = ", x0)
  print("x[i-1] = ", x[i-1])
  print("x[i] = ", x[i])
  print("y[i-1] = ", y[i-1])
  print("y[i] = ", y[i])

  if y[i] > 0.0:
    y0 = 10.0**( log10(y[i-1]) + (log10(y[i])-log10(y[i-1])) * (log10(x0)-log10(x[i-1]))/(log10(x[i])-log10(x[i-1])) )
  else:
    y0 = EPS

  print("y0 = ", y0)

  return y0

def lifetime(filename):

  D, N = read_sfd(filename)

  write_sfd(D, N, "lifetime.tmp1")

  N_gt_D = len(N)*[0]
  s = 0
  for i in range(len(N)-1,-1,-1):
    s += N[i]
    N_gt_D[i] = s
 
  write_sfd(D, N_gt_D, "lifetime.tmp2")

  P_i = 3.200e-18 * km**-2*yr**-1
  v = 5.698 * km

  f = open("lifetime.out", "w")
  f.write("# D d N_gt_D N_dt_d lifetime average_time_between_events Qstar\n")
  f.write("# km km 1 1 yr J/kg\n")

  for i in range(0,len(D)):
    Q = 1.0 * Qstar(D[i])
    d = D[i] * (2.0*Q/v**2)**(1./3.)
    N_gt_d = loginterp(D, N_gt_D, d)

    n1 = 1
    n2 = N_gt_d
    f_coll = P_i * D[i]**2/4.0 * n1 * n2
    P1 = 1./f_coll

    n1 = N_gt_D[i]
    n2 = N_gt_d
    f_coll = P_i * D[i]**2/4.0 * n1 * n2
    P2 = 1./f_coll

    f.write("%.8e %.8e  %.8e %.8e  %.8e %.8e  %.8e\n"% (D[i]/km, d/km, n1, n2, P1/yr, P2/yr, Q))

  f.close()

def main():
  lifetime("../001/tmp/0050.000.out")

if __name__ == "__main__":
  main()


